# HCT.pm (HCT) - the HCT environment. 
#
# Copyright 2008, 2009
# Slade Maurer
#
# This file is part of HCT, the Hardware Complexity Tool
#
# HCT is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# HCT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with HCT.  If not, see <http://www.gnu.org/licenses/>.

package HCT;

use strict;
use warnings;

use POSIX;

use HCT::Lang;
use HCT::HCT;
#use HCT::Shell;
use HCT::Constants;
use HCT::File;

our $NAME = 'The HDL Complexity Tool';
our $VERSION = "0.7.60";

# Global Variables: Output

our $BATCH = FALSE;
our $OUTPUT_AUTO = TRUE;
our $OUTPUT_FILE = undef;
our $OUTPUT_LEVEL = 1;
our $OUTPUT_FORMAT = 'TERMINAL';
our $OUTPUT_LOCK = FALSE;

#____________________________________________________________________
# Debugging

our $VERBOSE = TRUE;
our $DEBUG = FALSE;
our $PARSEDEBUG = FALSE;
our $LEXDEBUG = FALSE;

#____________________________________________________________________
# Threads

our $USE_THREADS = FALSE;
our $MAX_THREADS = 3;
our $THREAD_TIMEOUT = 0;

# Max threads
if (eval 'require Sys::CPU') {
    $MAX_THREADS = Sys::CPU::cpu_count ();
}

#____________________________________________________________________
# Shell

our $FRONTEND = 'HCT::Shell';
$HCT::Shell::STANDARD_PROMPT = 'hct> ';

#____________________________________________________________________
# Listing

our $START_DIR = getcwd (); # Start directory (if not defined, the current directory will be used).
our $LIST_RECURSIVELY = FALSE; # List subdirectories recursively.
our $LIST_ALL = FALSE; # List all.
our $LIST_SYS = FALSE;
our $LIST_TMP = FALSE;


#####################################################################
# Global Variables: HDL
#####################################################################

use vars qw(
	%FILE_ASSOCIATIONS
	$FORCE_LANGUAGE
);

%FILE_ASSOCIATIONS = EMPTY_LIST;

# Force language
$FORCE_LANGUAGE = UNKNOWN_LANGUAGE;

#____________________________________________________________________
# Global Subroutines

# Dumps some stuff!
# Uses Dumper from Data::Dumper as dump-function.

use Data::Dumper;

sub DUMP
{
    return Dumper (@_);
}

# Provides format of HCT debug or error messages.
# Formats:
#   filename:line: message
#   file: message

sub MESSAGE
{
	my ($package, $filename, $line, @message) = (shift, shift, shift, @_);
	$filename = get_basename ($filename);
	
	print STDERR "${filename}:" if defined $filename;
	print STDERR "${line}:"     if defined $line && defined $filename;
	print STDERR " "            if defined $filename;
	print STDERR @_;
}

# Handles debug messages. Prints message
# if $HCT::DEBUG set.

sub DEBUG
{
	return if $HCT::DEBUG == FALSE;
	HCT::MESSAGE ((caller), @_);
}

# Handles error messages. Prints message,
# then exit.

sub ERROR
{
	HCT::MESSAGE ((caller), @_);
	exit FAILURE;
}

# Handles messages from AUTOLOAD. Any OO module can use
# it to format output message.
#
# Skip methods: DESTROY.

sub AUTOLOAD
{
	my ($package, $filename, $line) = (shift, shift, shift);

	# our AUTOLOAD
	$_[0] =~ /^(.*?)\:\:([\w\d\_]+)$/;
	my ($from, $method ) = ($1, $2);
	
	if ($method eq 'DESTROY') {
		return;
	}
	
	HCT::MESSAGE (
		$package,
		$filename,
		$line,
		"$from has no method '$method'\n");
		
	return;	
}

1;

__END__

=head1 NAME

HCT - HCT environment. Provides global variables and subroutines. 

=head1 DESCRIPTION

=head1 GLOBAL VARIABLES

=head2 DEBUGGING

=over 4

=item B<DEBUG>

If true then debug messages will be available.

=item B<YYDEBUG>

Provides debugging parser. This variable connects with C<yydebug> parameter of the 
Parse::Yapp module, where can be found more details.

    Bit Value    Outputs
    0x01         Token reading (useful for Lexer debugging)
    0x02         States information
    0x04         Driver actions (shifts, reduces, accept...)
    0x08         Parse Stack dump
    0x10         Error Recovery tracing

=back

=head2 THREADS

=over 4

=item B<USE_THREADS>

If true, HCT will use threads.

=item B<MAX_THREADS>

The max number of threads.

=item B<THREAD_TIMEOUT>

Timeout for one thread.

=back

=head2 HDL

=over 4

=item B<FORCE_LANGUAGE>

If set, then all data will processed by this language.

=item B<FILE_ASSOCIATIONS>

List of file extensions and languages, which correspond to them.

	file extansion ==> language name

=back

=head2 LISTING

=over 4

=item B<BATCH>

If set, all files for processing will be located in a single list. 

=item B<START_DIR>

Keep a path to the current directory. 

=item B<LIST_RECURSIVELY>

If true, HCT will process subdirectories recursively.

=item B<LIST_ALL> 

If true, HCT will process all target files and directories (systems, temporary, etc.).

=back

=head1 GLOBAL SUBROUTINES

=over 4

=item B<DUMP>

Provides global C<Dumper> subroutine from L<Data::Dumper>.

=item B<MESSAGE>

Provides a standard of debug or error messages, that creates the HCT environment.

=item B<ERROR>

Produces an error message by using C<MESSAGE> subroutine, then exit.

=item B<DEBUG>

Produces a debug message (if C<DEBUG> variable set) by using C<MESSAGE> subroutine.

=item B<AUTOLOAD>

OO modules can use this subroutine to standardize the output messages. Standard 
example of using:

	sub AUTOLOAD
	{
		return HCT::AUTOLOAD (
			(caller),
			our $AUTOLOAD,
			@_
		);
	}

=back

=cut


